@@ -23,6 +23,10 @@ from django.db.models.sql.where import AND
 from django.db.transaction import TransactionManagementError
 from django.utils.encoding import force_str
 
+if django_version >= (4, 0):
+    from django.db.models.sql import Query
+    from django.db.backends.ddl_references import Expressions
+
 
 class Statement(DjStatement):
     def __hash__(self):
@@ -31,6 +35,14 @@ class Statement(DjStatement):
     def __eq__(self, other):
         return self.template == other.template and str(self.parts['name']) == str(other.parts['name'])
 
+    def rename_column_references(self, table, old_column, new_column):
+        for part in self.parts.values():
+            if hasattr(part, 'rename_column_references'):
+                part.rename_column_references(table, old_column, new_column)
+            condition = self.parts['condition']
+            if condition:
+                self.parts['condition'] = condition.replace(f'[{old_column}]', f'[{new_column}]')
+
 
 class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
 
@@ -70,7 +82,7 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             @sql_drop_constraint = 'ALTER TABLE [' + OBJECT_NAME(parent_object_id) + '] ' +
             'DROP CONSTRAINT [' + @sql_froeign_constraint_name + '] '
             FROM sys.foreign_keys
-            WHERE referenced_object_id = object_id('%(table)s')
+            WHERE referenced_object_id = object_id('%(table)s') and name = @sql_froeign_constraint_name
             exec sp_executesql @sql_drop_constraint
         END
         DROP TABLE %(table)s
@@ -161,11 +173,19 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         for fields in olds.difference(news):
             self._delete_composed_index(model, fields, {'unique': True}, self.sql_delete_index)
         # Created uniques
-        for fields in news.difference(olds):
-            columns = [model._meta.get_field(field).column for field in fields]
-            condition = ' AND '.join(["[%s] IS NOT NULL" % col for col in columns])
-            sql = self._create_unique_sql(model, columns, condition=condition)
-            self.execute(sql)
+        if django_version >= (4, 0):
+            for field_names in news.difference(olds):
+                fields = [model._meta.get_field(field) for field in field_names]
+                columns = [model._meta.get_field(field).column for field in field_names]
+                condition = ' AND '.join(["[%s] IS NOT NULL" % col for col in columns])
+                sql = self._create_unique_sql(model, fields, condition=condition)
+                self.execute(sql)
+        else:
+            for fields in news.difference(olds):
+                columns = [model._meta.get_field(field).column for field in fields]
+                condition = ' AND '.join(["[%s] IS NOT NULL" % col for col in columns])
+                sql = self._create_unique_sql(model, columns, condition=condition)
+                self.execute(sql)
 
     def _model_indexes_sql(self, model):
         """
@@ -182,12 +202,19 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             fields = [model._meta.get_field(field) for field in field_names]
             output.append(self._create_index_sql(model, fields, suffix="_idx"))
 
-        for field_names in model._meta.unique_together:
-            columns = [model._meta.get_field(field).column for field in field_names]
-            condition = ' AND '.join(["[%s] IS NOT NULL" % col for col in columns])
-            sql = self._create_unique_sql(model, columns, condition=condition)
-            output.append(sql)
-
+        if django_version >= (4, 0):
+            for field_names in model._meta.unique_together:
+                fields = [model._meta.get_field(field) for field in field_names]
+                columns = [model._meta.get_field(field).column for field in field_names]
+                condition = ' AND '.join(["[%s] IS NOT NULL" % col for col in columns])
+                sql = self._create_unique_sql(model, fields, condition=condition)
+                output.append(sql)
+        else:
+            for field_names in model._meta.unique_together:
+                columns = [model._meta.get_field(field).column for field in field_names]
+                condition = ' AND '.join(["[%s] IS NOT NULL" % col for col in columns])
+                sql = self._create_unique_sql(model, columns, condition=condition)
+                output.append(sql)
         for index in model._meta.indexes:
             if django_version >= (3, 2) and (
                 not index.contains_expressions or
@@ -198,14 +225,6 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                 output.append(index.create_sql(model, self))
         return output
 
-    def _alter_many_to_many(self, model, old_field, new_field, strict):
-        """Alter M2Ms to repoint their to= endpoints."""
-
-        for idx in self._constraint_names(old_field.remote_field.through, index=True, unique=True):
-            self.execute(self.sql_delete_index % {'name': idx, 'table': old_field.remote_field.through._meta.db_table})
-
-        return super()._alter_many_to_many(model, old_field, new_field, strict)
-
     def _db_table_constraint_names(self, db_table, column_names=None, unique=None,
                                    primary_key=None, index=None, foreign_key=None,
                                    check=None, type_=None, exclude=None):
@@ -345,7 +364,31 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                 self.execute(self._delete_constraint_sql(self.sql_delete_check, model, constraint_name))
         # Have they renamed the column?
         if old_field.column != new_field.column:
+            sql_restore_index = ''
+            # Drop unique indexes for table to be altered
+            index_names = self._db_table_constraint_names(model._meta.db_table, index=True)
+            for index_name in index_names:
+                if(index_name.endswith('uniq')):
+                    with self.connection.cursor() as cursor:
+                        cursor.execute(f"""
+                        SELECT COL_NAME(ic.object_id,ic.column_id) AS column_name,
+                               filter_definition
+                        FROM sys.indexes AS i
+                        INNER JOIN sys.index_columns AS ic
+                            ON i.object_id = ic.object_id AND i.index_id = ic.index_id
+                        WHERE i.object_id = OBJECT_ID('{model._meta.db_table}')
+                        and i.name = '{index_name}'
+                        """)
+                        result = cursor.fetchall()
+                        columns_to_recreate_index = ', '.join(['%s' % self.quote_name(column[0]) for column in result])
+                        filter_definition = result[0][1]
+                    sql_restore_index += f'CREATE UNIQUE INDEX {index_name} ON {model._meta.db_table} ({columns_to_recreate_index}) WHERE {filter_definition};'
+                    self.execute(self._db_table_delete_constraint_sql(
+                        self.sql_delete_index, model._meta.db_table, index_name))
             self.execute(self._rename_field_sql(model._meta.db_table, old_field, new_field, new_type))
+            # Restore indexes for altered table
+            if(sql_restore_index):
+                self.execute(sql_restore_index.replace(f'[{old_field.column}]', f'[{new_field.column}]'))
             # Rename all references to the renamed column.
             for sql in self.deferred_sql:
                 if isinstance(sql, DjStatement):
@@ -391,7 +434,17 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                     # Drop unique constraint, SQL Server requires explicit deletion
                     self._delete_unique_constraints(model, old_field, new_field, strict)
                     # Drop indexes, SQL Server requires explicit deletion
-                    self._delete_indexes(model, old_field, new_field)
+                    indexes_dropped = self._delete_indexes(model, old_field, new_field)
+                    if (
+                        new_field.get_internal_type() not in ("JSONField", "TextField") and
+                        (old_field.db_index or not new_field.db_index) and
+                        new_field.db_index or
+                        (indexes_dropped and sorted(indexes_dropped) == sorted(
+                            [index.name for index in model._meta.indexes]))
+                    ):
+                        create_index_sql_statement = self._create_index_sql(model, [new_field])
+                        if create_index_sql_statement.__str__() not in [sql.__str__() for sql in self.deferred_sql]:
+                            post_actions.append((create_index_sql_statement, ()))
         # Only if we have a default and there is a change from NULL to NOT NULL
         four_way_default_alteration = (
             new_field.has_default() and
@@ -452,7 +505,10 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                     )
                 )
             else:
-                self.execute(self._create_unique_sql(model, [new_field.column]))
+                if django_version >= (4, 0):
+                    self.execute(self._create_unique_sql(model, [new_field]))
+                else:
+                    self.execute(self._create_unique_sql(model, [new_field.column]))
         # Added an index?
         # constraint will no longer be used in lieu of an index. The following
         # lines from the truth table show all True cases; the rest are False:
@@ -480,13 +536,24 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                         )
                     )
                 else:
-                    self.execute(self._create_unique_sql(model, columns=[old_field.column]))
+                    if django_version >= (4, 0):
+                        self.execute(self._create_unique_sql(model, [old_field]))
+                    else:
+                        self.execute(self._create_unique_sql(model, columns=[old_field.column]))
             else:
-                for fields in model._meta.unique_together:
-                    columns = [model._meta.get_field(field).column for field in fields]
-                    if old_field.column in columns:
-                        condition = ' AND '.join(["[%s] IS NOT NULL" % col for col in columns])
-                        self.execute(self._create_unique_sql(model, columns, condition=condition))
+                if django_version >= (4, 0):
+                    for field_names in model._meta.unique_together:
+                        columns = [model._meta.get_field(field).column for field in field_names]
+                        fields = [model._meta.get_field(field) for field in field_names]
+                        if old_field.column in columns:
+                            condition = ' AND '.join(["[%s] IS NOT NULL" % col for col in columns])
+                            self.execute(self._create_unique_sql(model, fields, condition=condition))
+                else:
+                    for fields in model._meta.unique_together:
+                        columns = [model._meta.get_field(field).column for field in fields]
+                        if old_field.column in columns:
+                            condition = ' AND '.join(["[%s] IS NOT NULL" % col for col in columns])
+                            self.execute(self._create_unique_sql(model, columns, condition=condition))
             # Restore indexes
             index_columns = []
             if old_field.db_index and new_field.db_index:
@@ -498,7 +565,12 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                         index_columns.append(columns)
             if index_columns:
                 for columns in index_columns:
-                    self.execute(self._create_index_sql(model, columns, suffix='_idx'))
+                    create_index_sql_statement = self._create_index_sql(model, columns)
+                    if (create_index_sql_statement.__str__()
+                        not in [sql.__str__() for sql in self.deferred_sql] + [statement[0].__str__() for statement in post_actions]
+                        ):
+                        self.execute(create_index_sql_statement)
+
         # Type alteration on primary key? Then we need to alter the column
         # referring to us.
         rels_to_update = []
@@ -525,6 +597,11 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             fragment, other_actions = self._alter_column_type_sql(
                 new_rel.related_model, old_rel.field, new_rel.field, rel_type
             )
+            # Drop related_model indexes, so it can be altered
+            index_names = self._db_table_constraint_names(old_rel.related_model._meta.db_table, index=True)
+            for index_name in index_names:
+                self.execute(self._db_table_delete_constraint_sql(
+                    self.sql_delete_index, old_rel.related_model._meta.db_table, index_name))
             self.execute(
                 self.sql_alter_column % {
                     "table": self.quote_name(new_rel.related_model._meta.db_table),
@@ -534,6 +611,8 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             )
             for sql, params in other_actions:
                 self.execute(sql, params)
+            # Restore related_model indexes
+            self.execute(self._create_index_sql(new_rel.related_model, [new_rel.field]))
         # Does it have a foreign key?
         if (new_field.remote_field and
                 (fks_dropped or not old_field.remote_field or not old_field.db_constraint) and
@@ -576,8 +655,11 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
 
     def _delete_indexes(self, model, old_field, new_field):
         index_columns = []
+        index_names = []
         if old_field.db_index and new_field.db_index:
             index_columns.append([old_field.column])
+        elif old_field.null != new_field.null:
+            index_columns.append([old_field.column])
         for fields in model._meta.index_together:
             columns = [model._meta.get_field(field).column for field in fields]
             if old_field.column in columns:
@@ -592,6 +674,7 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                 index_names = self._constraint_names(model, columns, index=True)
                 for index_name in index_names:
                     self.execute(self._delete_constraint_sql(self.sql_delete_index, model, index_name))
+        return index_names
 
     def _delete_unique_constraints(self, model, old_field, new_field, strict=False):
         unique_columns = []
@@ -681,43 +764,99 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         if self.connection.features.connection_persists_old_columns:
             self.connection.close()
 
-    def _create_unique_sql(self, model, columns, name=None, condition=None, deferrable=None, include=None, opclasses=None):
-        if (deferrable and not getattr(self.connection.features, 'supports_deferrable_unique_constraints', False)):
-            return None
+    if django_version >= (4, 0):
+        def _create_unique_sql(self, model, fields,
+                               name=None, condition=None, deferrable=None,
+                               include=None, opclasses=None, expressions=None):
+            if (deferrable and not getattr(self.connection.features, 'supports_deferrable_unique_constraints', False) or
+                (condition and not self.connection.features.supports_partial_indexes) or
+                (include and not self.connection.features.supports_covering_indexes) or
+                    (expressions and not self.connection.features.supports_expression_indexes)):
+                return None
 
-        def create_unique_name(*args, **kwargs):
-            return self.quote_name(self._create_index_name(*args, **kwargs))
+            def create_unique_name(*args, **kwargs):
+                return self.quote_name(self._create_index_name(*args, **kwargs))
 
-        table = Table(model._meta.db_table, self.quote_name)
-        if name is None:
-            name = IndexName(model._meta.db_table, columns, '_uniq', create_unique_name)
-        else:
-            name = self.quote_name(name)
-        columns = Columns(table, columns, self.quote_name)
-        statement_args = {
-            "deferrable": self._deferrable_constraint_sql(deferrable)
-        } if django_version >= (3, 1) else {}
-        include = self._index_include_sql(model, include) if django_version >=(3, 2) else ''
+            compiler = Query(model, alias_cols=False).get_compiler(connection=self.connection)
+            columns = [field.column for field in fields]
+            table = model._meta.db_table
 
-        if condition:
-            return Statement(
-                self.sql_create_unique_index,
-                table=table,
-                name=name,
-                columns=columns,
-                condition=' WHERE ' + condition,
-                **statement_args,
-                include=include,
-            ) if self.connection.features.supports_partial_indexes else None
-        else:
-            return Statement(
-                self.sql_create_unique,
-                table=table,
-                name=name,
-                columns=columns,
-                **statement_args,
-                include=include,
-            )
+            if name is None:
+                name = IndexName(table, columns, '_uniq', create_unique_name)
+            else:
+                name = self.quote_name(name)
+
+            if columns:
+                columns = self._index_columns(table, columns, col_suffixes=(), opclasses=opclasses)
+            else:
+                columns = Expressions(table, expressions, compiler, self.quote_value)
+            statement_args = {
+                "deferrable": self._deferrable_constraint_sql(deferrable)
+            }
+            include = self._index_include_sql(model, include)
+
+            if condition:
+                return Statement(
+                    self.sql_create_unique_index,
+                    table=self.quote_name(table),
+                    name=name,
+                    columns=columns,
+                    condition=' WHERE ' + condition,
+                    **statement_args,
+                    include=include,
+                ) if self.connection.features.supports_partial_indexes else None
+            else:
+                return Statement(
+                    self.sql_create_unique,
+                    table=self.quote_name(table),
+                    name=name,
+                    columns=columns,
+                    **statement_args,
+                    include=include,
+                )
+    else:
+        def _create_unique_sql(self, model, columns,
+                               name=None, condition=None, deferrable=None,
+                               include=None, opclasses=None, expressions=None):
+            if (deferrable and not getattr(self.connection.features, 'supports_deferrable_unique_constraints', False) or
+                (condition and not self.connection.features.supports_partial_indexes) or
+                (include and not self.connection.features.supports_covering_indexes) or
+                    (expressions and not self.connection.features.supports_expression_indexes)):
+                return None
+
+            def create_unique_name(*args, **kwargs):
+                return self.quote_name(self._create_index_name(*args, **kwargs))
+
+            table = Table(model._meta.db_table, self.quote_name)
+            if name is None:
+                name = IndexName(model._meta.db_table, columns, '_uniq', create_unique_name)
+            else:
+                name = self.quote_name(name)
+            columns = Columns(table, columns, self.quote_name)
+            statement_args = {
+                "deferrable": self._deferrable_constraint_sql(deferrable)
+            } if django_version >= (3, 1) else {}
+            include = self._index_include_sql(model, include) if django_version >= (3, 2) else ''
+
+            if condition:
+                return Statement(
+                    self.sql_create_unique_index,
+                    table=self.quote_name(table) if isinstance(table, str) else table,
+                    name=name,
+                    columns=columns,
+                    condition=' WHERE ' + condition,
+                    **statement_args,
+                    include=include,
+                ) if self.connection.features.supports_partial_indexes else None
+            else:
+                return Statement(
+                    self.sql_create_unique,
+                    table=self.quote_name(table) if isinstance(table, str) else table,
+                    name=name,
+                    columns=columns,
+                    **statement_args,
+                    include=include,
+                )
 
     def _create_index_sql(self, model, fields, *, name=None, suffix='', using='',
                           db_tablespace=None, col_suffixes=(), sql=None, opclasses=(),
@@ -799,10 +938,14 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
 
         # Add any unique_togethers (always deferred, as some fields might be
         # created afterwards, like geometry fields with some backends)
-        for fields in model._meta.unique_together:
-            columns = [model._meta.get_field(field).column for field in fields]
+        for field_names in model._meta.unique_together:
+            fields = [model._meta.get_field(field) for field in field_names]
+            columns = [model._meta.get_field(field).column for field in field_names]
             condition = ' AND '.join(["[%s] IS NOT NULL" % col for col in columns])
-            self.deferred_sql.append(self._create_unique_sql(model, columns, condition=condition))
+            if django_version >= (4, 0):
+                self.deferred_sql.append(self._create_unique_sql(model, fields, condition=condition))
+            else:
+                self.deferred_sql.append(self._create_unique_sql(model, columns, condition=condition))
 
         constraints = [constraint.constraint_sql(model, self) for constraint in model._meta.constraints]
         # Make the table
@@ -828,7 +971,7 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
 
     def _delete_unique_sql(
         self, model, name, condition=None, deferrable=None, include=None,
-        opclasses=None,
+        opclasses=None, expressions=None
     ):
         if (
             (
@@ -836,13 +979,15 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                 not self.connection.features.supports_deferrable_unique_constraints
             ) or
             (condition and not self.connection.features.supports_partial_indexes) or
-            (include and not self.connection.features.supports_covering_indexes)
+            (include and not self.connection.features.supports_covering_indexes) or
+            (expressions and not self.connection.features.supports_expression_indexes)
         ):
             return None
         if condition or include or opclasses:
             sql = self.sql_delete_index
             with self.connection.cursor() as cursor:
-                cursor.execute("SELECT 1 FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = '%s'" % name)
+                cursor.execute(
+                    "SELECT 1 FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE WHERE CONSTRAINT_NAME = '%s'" % name)
                 row = cursor.fetchone()
                 if row:
                     sql = self.sql_delete_unique
@@ -974,3 +1119,14 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             raise NotImplementedError("The backend does not support %s conditions on unique constraint %s." %
                                       (constraint.condition.connector, constraint.name))
         super().add_constraint(model, constraint)
+
+    def _collate_sql(self, collation):
+        return ' COLLATE ' + collation
+
+    def _create_index_name(self, table_name, column_names, suffix=""):
+        index_name = super()._create_index_name(table_name, column_names, suffix)
+        # Check if the db_table specified a user-defined schema
+        if('].[' in index_name):
+            new_index_name = index_name.replace('[', '').replace(']', '').replace('.', '_')
+            return new_index_name
+        return index_name
