@@ -10,18 +10,21 @@ from mssql.base import DatabaseWrapper
 from ..models import (
     Author,
     Editor,
+    M2MOtherModel,
     Post,
     TestUniqueNullableModel,
     TestNullableUniqueTogetherModel,
+    TestRenameManyToManyFieldModel,
 )
 
 
 @skipUnlessDBFeature('supports_nullable_unique_constraints')
 class TestNullableUniqueColumn(TestCase):
     def test_multiple_nulls(self):
-        # Issue #45 (case 1) - after field `x` has had its type changed, the filtered UNIQUE
-        # INDEX which is implementing the nullable unique constraint should still be correctly
-        # in place - i.e. allowing multiple NULLs but still enforcing uniqueness of non-NULLs
+        # Issue https://github.com/ESSolutions/django-mssql-backend/issues/45 (case 1)
+        # After field `x` has had its type changed, the filtered UNIQUE INDEX which is
+        # implementing the nullable unique constraint should still be correctly in place
+        # i.e. allowing multiple NULLs but still enforcing uniqueness of non-NULLs
 
         # Allowed
         TestUniqueNullableModel.objects.create(x=None, test_field='randomness')
@@ -50,8 +53,9 @@ class TestPartiallyNullableUniqueTogether(TestCase):
             Post.objects.create(title="foo", author=author, alt_editor=editor)
 
     def test_after_type_change(self):
-        # Issue #45 (case 2) - after one of the fields in the `unique_together` has had its
-        # type changed in a migration, the constraint should still be correctly enforced
+        # Issue https://github.com/ESSolutions/django-mssql-backend/issues/45 (case 2)
+        # After one of the fields in the `unique_together` has had its type changed
+        # in a migration, the constraint should still be correctly enforced
 
         # Multiple rows with a=NULL are considered different
         TestNullableUniqueTogetherModel.objects.create(a=None, b='bbb', c='ccc')
@@ -63,6 +67,21 @@ class TestPartiallyNullableUniqueTogether(TestCase):
             TestNullableUniqueTogetherModel.objects.create(a='aaa', b='bbb', c='ccc')
 
 
+class TestRenameManyToManyField(TestCase):
+    def test_uniqueness_still_enforced_afterwards(self):
+        # Issue https://github.com/microsoft/mssql-django/issues/86
+        # Prep
+        thing1 = TestRenameManyToManyFieldModel.objects.create()
+        other1 = M2MOtherModel.objects.create(name='1')
+        other2 = M2MOtherModel.objects.create(name='2')
+        thing1.others_renamed.set([other1, other2])
+        # Check that the unique_together on the through table is still enforced
+        ThroughModel = TestRenameManyToManyFieldModel.others_renamed.through
+        with self.assertRaises(IntegrityError, msg='Through model fails to enforce uniqueness after m2m rename'):
+            # This should fail due to the unique_together because (thing1, other1) is already in the through table
+            ThroughModel.objects.create(testrenamemanytomanyfieldmodel=thing1, m2mothermodel=other1)
+
+
 class TestUniqueConstraints(TransactionTestCase):
     def test_unsupportable_unique_constraint(self):
         # Only execute tests when running against SQL Server
